package syncer

import (
	"context"
	"fmt"
	"log"
	"os"
	"os/exec"
	"path"
	"sync"

	"git.acooldomain.co/acoolname/git-syncer/src/enumerators"
)

type ContextKey string

var RepositoryContextKey ContextKey = "Repository"

type SyncResult struct {
	Endpoint *enumerators.RepositoryEndpoint
	Err      error
}

func syncRepo(ctx context.Context, location string, repository *enumerators.RepositoryEndpoint) error {
	select {
	case <-ctx.Done():
		return ctx.Err()
	default:
	}

	err := os.MkdirAll(location, 0760)
	if err != nil {
		return err
	}

	stat, err := os.Stat(path.Join(location, ".git"))
	if err != nil {
		if os.IsNotExist(err) {
			command := exec.CommandContext(ctx, "git", "clone", repository.Url, location)
			err := command.Run()
			if err != nil {
				output, _ := command.CombinedOutput()
				log.Printf("Failed to clone repo \"%s\": %v %#v ", repository.Url, err, output)
				return err
			}
		}
	} else {
		if !stat.IsDir() {
			return fmt.Errorf(".git exists and is not a directory")
		}
	}

	if repository.Pull {
		command := exec.CommandContext(ctx, "git", "fetch", "--all")
		command.Dir = location
		err = command.Run()
		if err != nil {
			output, _ := command.CombinedOutput()
			log.Printf("Failed to pull repo \"%s\": %v %#v ", repository.Url, err, output)
			return err
		}

	}
	if repository.Push {
		command := exec.CommandContext(ctx, "git", "push", "origin", "--all")
		command.Dir = location
		err = command.Run()
		if err != nil {
			output, _ := command.CombinedOutput()
			log.Printf("Failed to push repo \"%s\": %v %#v ", repository.Url, err, output)

			return err
		}
	}

	return nil
}

func StoreRepositories(ctx context.Context, baseLocation string, repositories chan *enumerators.RepositoryEndpoint, locationOverrides map[string]string) {
	wg := sync.WaitGroup{}
	results := make(chan SyncResult)
	for repoEndpoint := range repositories {
		wg.Add(1)
		location, ok := locationOverrides[repoEndpoint.Url]
		if !ok {
			location = path.Join(baseLocation, repoEndpoint.FullName)
		}

		go func(r *enumerators.RepositoryEndpoint) {
			defer wg.Done()
			err := syncRepo(ctx, location, r)
			results <- SyncResult{
				Endpoint: repoEndpoint,
				Err:      err,
			}
		}(repoEndpoint)
	}

	go func() {
		wg.Wait()
		close(results)
	}()

	for res := range results {
		if res.Err != nil {
			log.Printf("Failed to sync repo \"%s\": %v ", res.Endpoint.Url, res.Err)
		}
	}
}
